using DFTK
using PyCall

if !("ASE_ESPRESSO_COMMAND" in keys(ENV))
    ENV["ASE_ESPRESSO_COMMAND"] = "/usr/bin/pw.x -in PREFIX.pwi > PREFIX.pwo"
end

cp("./qe_pseudos", "/tmp", force=true)
ENV["ESPRESSO_PSEUDO"] = "/tmp/qe_pseudos"

const PSEUDOMAP = Dict(
   "B"  => "B.pbe-hgh.UPF",
   "Hf" => "Hf.pbe-sp-hgh.UPF",
)

const Ry = 0.5
const eV = 0.03674932248

"""Ecut is in Hartree, tol is energy tolerance in Hartree, kspacing is in inverse Bohrs"""
function run_qe(lattice, atoms; Ecut=20, tol=1e-11, mixing_mode="plain",
                fileprefix="espresso", kspacing=0.3, smearing="gaussian",
                temperature=0, mixing_beta=0.7, mixing_ndim=10,
                maxiter=100, kgrid=nothing, magnetic_moments=nothing, input_dft="pbe", kwargs...)
    if isnothing(kgrid)
        kgrid = kgrid_size_from_minimal_spacing(lattice, kspacing)
        @info "Autoselected kgrid_size", kgrid
    end
    system = ase_atoms(lattice, atoms, magnetic_moments)
    system.calc = pyimport("ase.calculators.espresso").Espresso(
        label=fileprefix,
        input_dft=input_dft,
        pseudopotentials=PSEUDOMAP,
        kpts=kgrid,
        ecutwfc=Ecut / Ry,  # QE uses Rydbergs
        tstress=true,       # Compute stresses
        tprnfor=true,       # Compute forces
        mixing_mode=mixing_mode,
        mixing_beta=mixing_beta,
        conv_thr=tol / Ry,
        occupations=ifelse(temperature > 0, "smearing", "fixed"),
        smearing=smearing,
        degauss=temperature / Ry,
        electron_maxstep=maxiter,
        mixing_ndim=mixing_ndim,
        # verbosity="high",
        kwargs...
    )
    system.get_potential_energy() * eV
end

#
#
#

structure = "HfB2_mp-1994_primitive.cif"  # Downloaded from https://materialsproject.org/materials/mp-1994
lattice = load_lattice(structure)
atoms   = load_atoms(structure)
atoms   = [ElementPsp(el.symbol, psp=load_psp(el.symbol, functional="pbe")) => position
           for (el, position) in atoms]

tol = 1e-8
kgrid = [6, 6, 6]     # k-point grid (Regular Monkhorst-Pack grid)
Ecut  = 50
temperature = 0.01
run_qe(lattice, atoms, Ecut=Ecut, tol=tol, fileprefix="qe/espresso",
       kgrid=kgrid, temperature=temperature, mixing_beta=0.8)
